function [c,Ls]=mdct(f,g,M,L)
%MDCT  Modified Discrete Cosine Transform
%   Usage:  c=mdct(f,g,M);
%           c=mdct(f,g,M,L);
%           [c,Ls]=mdct(f,g,M);
%           [c,Ls]=mdct(f,g,M,L);
%
%   Input parameters:
%         f     : Input data
%         g     : Window function.
%         M     : Number of bands.
%         L     : Length of transform to do.
%   Output parameters:
%         c     : 2*M x N array of coefficients.
%         Ls    : Length of input signal.
%
%   MDCT(f,g,M) computes a Modified Discrete Cosine Transform with M bands
%   and window g.
%
%   The length of the transform will be the smallest possible that is
%   larger than the signal. f will be zero-extended to the length of the 
%   transform. If f is a matrix, the transformation is applied to each column.
%   g must be whole-point even.
%
%   MDCT(f,g,M,L) computes the MDCT transform as above, but does
%   a transform of length L. f will be cut or zero-extended to length L
%   before the transform is done.
%
%   The window g may be a text string decribing the window. The following
%   types are recognized
% 
%   'gauss'     - Gaussian window.
%
%   'dualgauss' - Canonical dual of Gaussian window.
%
%   'tight'     - Tight window generated from a Gaussian.
%
%   'sqrthann'  - Square root of Hanning window. This is a tight, FIR
%                 window.  
% 
%   [c,Ls]=MDCT(f,g,M) or [c,Ls]=MDCT(f,g,M,L) additionally return the
%   length of the input signal f. This is handy for reconstruction:
% 
%            [c,Ls]=mdct(f,g,M);
%            fr=imdct(c,gd,M,Ls);
% 
%   will reconstuct the signal f no matter what the length of f is, provided
%   that gd is a dual Wilson window of g.
%
%   The MDCT is sometimes known as an odd-stacked cosine modulated filter
%   bank. The MDCT defined by this routine is slightly different from the
%   most common definition of the MDCT, in order to be able to use the
%   same window functions as the Wilson transform.
%
%   Assume that the following code has been executed for a column vector f
%   of length L:
% 
%            c=mdct(f,g,M);  % Compute the MDCT of f.
%            N=size(c,2);    % Number of translation coefficients.
% 
%   The following holds for m=0,...,M-1 
%   and n=0,...,N-1:
%
%   If m+n is even:
% 
%                     L-1
%        c(m+1,n+1) = sum f(l+1)*cos(pi*(m+.5)*l/M+pi/4)*g(l-n*M+1)
%                     l=0
% 
%   If m+n is odd:
%                     L-1
%        c(m+1,n+1) = sum f(l+1)*sin(pi*(m+.5)*l/M+pi/4)*g(l-n*M+1)
%                     l=0
% 
%   SEE ALSO:  IMDCT, DWILT, WILDUAL, WILORTH
% 
%   REFERENCES:
%     H. B?lcskei and F. Hlawatsch. Oversampled Wilson-type cosine modulated
%     filter banks with linear phase. In Asilomar Conf. on Signals, Systems,
%     and Computers, pages 998-1002, nov 1996.
%     
%     H. S. Malvar. Signal Processing with Lapped Transforms. Artech House
%     Publishers, 1992.
%     
%     J. P. Princen and A. B. Bradley. Analysis/synthesis filter bank design
%     based on time domain aliasing cancellation. IEEE Transactions on
%     Acoustics, Speech, and Signal Processing, ASSP-34(5):1153-1161, 1986.
%     
%     J. P. Princen, A. W. Johnson, and A. B. Bradley. Subband/transform
%     coding using filter bank designs based on time domain aliasing
%     cancellation. Proceedings - ICASSP, IEEE International Conference on
%     Acoustics, Speech and Signal Processing, pages 2161-2164, 1987.

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR:    Peter Soendergaard
%   TESTING:   TEST_MDCT
%   REFERENCE: REF_MDCT

error(nargchk(3,4,nargin));

if nargin<4
  L=[];
end;

%assert_squarelat(M,M,1,'MDCT',0);

if isnumeric(g)
  if size(g,2)>1
    if size(g,1)>1
      error('g must be a vector');
    else
      % g was a row vector.
      g=g(:);
    end;
  end;
  Lwindow=size(g,1);
else
  Lwindow=0;
end;

% Change f to correct shape.
[f,Ls,W,wasrow,remembershape]=comp_sigreshape_pre(f,'MDCT',0);

[b,N,L]=assert_L(Ls,Lwindow,L,2*M,'MDCT');

g=comp_window(g,M,2*M,L,1,'MDCT');

f=postpad(f,L);  

% Call the computational subroutines.
a=M;

c2 = comp_gdgt(f,g,a,2*M,L,0,.5,0,0);
c  = comp_dwiltiii(c2,a);

% Get correct shape.
c=reshape(c,M,N,W);

% Clean coefficients if they are known to be real
% If input is real, and window is real, output must be real as well.
if (isreal(g) && isreal(f));
  c=real(c);
end;